package Brainboxes::AsciiIo;
# Yes, the package name is terrible: but the 'ASCII command set' widely used for remote I/O modules doesn't have a proper name...

use IO::Socket;
use Time::HiRes qw/time sleep/;
use Carp;

use strict;
use warnings;
      
sub new {
    my $class = shift @_;
    my %args = ref $_[0] ? %{$_[0]} : @_; # allow args to be in a hash ref or just a list

    my $self = {
        timeout => $args{timeout},
        rxd => q{},
    };
    bless $self, $class;
    
    $self->{timeout} = 5 if !defined $self->{timeout};
    
    if (defined $args{ipaddr}) {
        my $ipport = $args{ipport} || 9500;
        $self->{port} = IO::Socket::INET->new(
            Proto     => 'tcp',
            PeerAddr  => $args{ipaddr},
            PeerPort  => $ipport,
          );
        croak "Cannot connect to TCP server $args{ipaddr}:$ipport" if !$self->{port};
        binmode $self->{port};

        $self->{selectmask} = q{};
        vec($self->{selectmask},fileno($self->{port}),1) = 1;

    }
    else {
        croak "${class}->new() needs an 'ipaddr' argument";
    }
    
    return $self;
}
    
sub command_noresponse { # only use this method for commands which should give no response, e.g. '#**' or '~**'
    my ($self, $txdata) = @_;

    my $remote = $self->{port};
    print {$remote} $txdata."\r"; 
    binmode $remote; # force immediate transmission of the data
    return q{};
}
    
sub command_response {
    my ($self, $txdata) = @_;

    my $remote = $self->{port};
    print {$remote} $txdata."\r"; 
    binmode $remote; # force immediate transmission of the data
    my $tstart = time;
    
    #~ print "Discarding data in rx buffer before query: '$self->{rxd}'\n" if $self->{rxd} !~ /\A [\r\n]* \z/xms;
    $self->{rxd} = q{};
    
    while (1) {
        if (select(my $rout = $self->{selectmask}, undef, undef, 0.5)) {
            # receive IP data
            my $newdata;
            recv($remote, $newdata, 1024, 0);
            croak "Connection to server has been broken" if $newdata eq '';
            $self->{rxd} .= $newdata;
            if ($self->{rxd} =~ /\A [\r\n]* (.*?) [\r\n]+ (.*) \z/xms) {
                my $resp = $1;
                $self->{rxd} = $2;
                return $resp;
            }
        }
        return undef if time > $tstart + $self->{timeout};
    }
}

1;